/*
 * Copyright (C) 2010 Guillaume Quintin.
 * This file is part of cct.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/*

From ECMA-48 or ISO/IEC 6429:
=============================


Control sequences

A control sequence is a string of bit combinations starting with the
control function CONTROL SEQUENCE INTRODUCER (CSI) followed by one or
more bit combinations representing parameters, if any, and by one or
more bit combinations identifying the control function. The control
function CSI itself is an element of the C1 set. The format of a control
sequence is

CSI P ... P I ... I F

where

a) CSI is represented by bit combinations 01/11 (representing ESC)
and 05/11 in a 7-bit code or by bit combination 09/11 in an 8-bit code,
see 5.3;

b) P ... P are Parameter Bytes, which, if present, consist of bit
combinations from 03/00 to 03/15;

c) I ... I are Intermediate Bytes, which, if present, consist of bit
combinations from 02/00 to 02/15.

Together with the Final Byte F, they identify the control function;

NOTE

The number of Intermediate Bytes is not limited by this Standard; in
practice, one Intermediate Byte will be sufficient since with sixteen
different bit combinations available for the Intermediate Byte over one
thousand control functions may be identified.

d) F is the Final Byte; it consists of a bit combination from 04/00 to
07/14; it terminates the control sequence and together with the
Intermediate Bytes, if present, identifies the control function. Bit
combinations 07/00 to 07/14 are available as Final Bytes of control
sequences for private (orexperimental) use.

[...]

Independent control functions

These control functions are represented in 7-bit and 8-bit codes by
2-character escape sequences of the form ESC Fs, where ESC is represented
by bit combination 01/11 and Fs is represented by a bit combination from
06/00 to 07/14.

*/

#define BUF_LEN  256

#define GETC(c)  { if ( read(fd,&c,1) <= 0 ) return 0; }
#define ESC      27
char *cct_input(int fd) {
  int i;
  char c;
  static char buf[BUF_LEN];

  GETC(c);
  if ( c != ESC ) {
    buf[0] = c;
    buf[1] = 0;
    return buf;
  }
  GETC(c);
  /* Independant control function */
  if ( c >= 0x60 && c <= 0x7E ) {
    buf[1] = c;
    buf[2] = 0;
    return buf;
  }
  /* We do not have a CSI */
  if ( c != '[' ) return 0;
  /* We have a CSI */
  buf[0] = ESC;
  buf[1] = '[';
  /* Get the Parameter bytes */
  for( i = 2 ; i < BUF_LEN ; i++ ) {
    GETC(c);
    buf[i] = c;
    /* We encounter an Intermediate bytes */
    if ( c >= 0x20 && c <= 0x2F ) break;
    /* We encounter a Final byte */
    if ( c >= 0x40 && c <= 0x7E ) {
      buf[i + 1] = 0;
      return buf;
    }
    /* We have an invalid Parameter byte */
    if ( c < 0x30 || c > 0x3F ) return 0;
  }
  /* Get the Internediate bytes */
  for( ; i < BUF_LEN ; i++ ) {
    GETC(c);
    buf[i] = c;
    /* We encounter a Final byte */
    if ( c >= 0x40 && c <= 0x7E ) {
      buf[i + 1] = 0;
      return buf;
    }
    /* We have an invalid Intermediate byte */
    if ( c < 0x20 || c > 0x2F ) return 0;
  }
  /* If we get here, this means that the Control Sequence
     is correct but is too lengthy, this is unlikely to happen
     as specified in the standard, so we return NULL */
  return 0;
}
#undef ESC
#undef GETC

char *cct_helper_print_input(const char *c) {
  int i;
  static char buf[BUF_LEN * 2];
  for( i = 0 ; c[0] && i < (BUF_LEN * 2) ; c++ ) {
    if ( c[0] >= 0x01 && c[0] <= 0x1A ) {
      buf[i] = '^';
      i++;
      buf[i] = c[0] + 'A' - 1;
      i++;
      continue;
    }
    if ( c[0] == 0x1B ) {
      buf[i] = '^';
      i++;
      buf[i] = '[';
      i++;
      continue;
    }
    if ( (c[0] >= 0x1C && c[0] <= 0x1F) || c[0] == 0x7F ) continue;
    buf[i] = c[0];
    i++;
  }
  buf[i] = 0;
  return buf;
}

#undef BUF_LEN

/*

This function is here to help programmers "know" which key
was pressed. There is no standard (to my current knowledge)
defining a mapping between keyboards keys and control sequences.
The user can use the terminfo database to get this mapping.
However, it seems that on most terminal emulators, some keys
always produce the same control sequences like the arrow keys.

*/

int cct_get_key(const char *c) {
  if ( !c ) return 0;
  if ( c[1] == 0 ) return (int)c[0];
  if ( !strcmp(c,"\033[A") ) return CCT_KEY_UP;
  if ( !strcmp(c,"\033[B") ) return CCT_KEY_DOWN;
  if ( !strcmp(c,"\033[C") ) return CCT_KEY_RIGHT;
  if ( !strcmp(c,"\033[D") ) return CCT_KEY_LEFT;
  return 0;
}

