/*
 * Copyright (C) 2010 Guillaume Quintin.
 * This file is part of cct.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define TEXT_LEN 256
int cct_textbox_init(cct_textbox *txt,
                     terminal *t,int x,int y,int w,int maxlen)
{
  if ( x < 0 || x >= t->w || x + w - 1 >= t->w ||
       y < 0 || y >= t->h ) return -1;
  txt->text = malloc(TEXT_LEN);
  if ( !txt->text ) return -1;
  txt->t = t;
  txt->x = x;
  txt->y = y;
  txt->w = w;
  txt->max = TEXT_LEN;
  txt->first = 0;
  txt->last = 0;
  txt->sel = 0;
  txt->maxlen = maxlen;
  return 0;
}
#undef TEXT_LEN

void cct_textbox_kill(cct_textbox *txt) {
  free(txt->text);
}

static void cct_textbox_movefirst(cct_textbox *txt) {
  if ( txt->sel < 0 ) txt->sel = 0;
  if ( txt->sel > txt->last ) txt->sel = txt->last;
  /*  0123
   *  abcX
   * |....| => l = 4
   */
  if ( txt->last < txt->w ) {
    txt->first = 0;
    return;
  }
  /* 0123456789
   * abcdefghij
   *        |....| => l = 4
   */
  if ( txt->first + txt->w -1  > txt->last ) {
    txt->first = txt->last - txt->w + 1;
  }
  /* 0123456789
   * abcdefghik
   *   ^ |....| => l = 4
   *   |
   *  sel
   */
  if ( txt->sel < txt->first ) {
    txt->first = txt->sel;
    return;
  }
  /* 0123456789
   * abcdefghik
   *  |....| ^  => l = 4
   *         |
   *        sel
   */
  if ( txt->sel >= (txt->first + txt->w) ) {
    txt->first = txt->sel - txt->w + 1;
    return;
  }
  /* do nothing here */
}

void cct_textbox_resize(cct_textbox *txt,int x,int y,int w) {
  if ( x < 0 || x >= txt->t->w || x + w - 1 >= txt->t->w ||
       y < 0 || y >= txt->t->h ) return;
  txt->x = x;
  txt->y = y;
  if ( w == txt->w ) return;
  txt->w = w;
  cct_textbox_movefirst(txt);
}

void cct_textbox_clear(cct_textbox *txt) {
  txt->last = 0;
  txt->text[0] = 0;
  cct_textbox_movefirst(txt);
}

void cct_textbox_home(cct_textbox *txt) {
  txt->sel = 0;
  cct_textbox_movefirst(txt);
}

void cct_textbox_end(cct_textbox *txt) {
  txt->sel = txt->last;
  cct_textbox_movefirst(txt);
}

void cct_textbox_next(cct_textbox *txt) {
  txt->sel++;
  cct_textbox_movefirst(txt);
}

void cct_textbox_prev(cct_textbox *txt) {
  txt->sel--;
  cct_textbox_movefirst(txt);
}

void cct_textbox_del(cct_textbox *txt) {
  int i;
  if ( txt->sel < txt->last ) {
    for( i = txt->sel ; i < txt->last ; i++ ) {
      txt->text[i] = txt->text[i + 1];
    }
    txt->last--;
    cct_textbox_movefirst(txt);
  }
}

void cct_textbox_bs(cct_textbox *txt) {
  if ( txt->sel == 0 ) return;
  txt->sel--;
  cct_textbox_del(txt);
}

int cct_textbox_insert(cct_textbox *txt,const char *s) {
  int l,i,n;
  l = strlen(s);
  if ( l == 0 ) return 0;
  if ( txt->maxlen && txt->last + l > txt->maxlen ) return 0;
  n = txt->last + 1 + l;
  if ( n > txt->max ) {
    char *tmp;
    for( ; txt->max < n ; txt->max *= 2 );
    tmp = realloc(txt->text,txt->max);
    if ( !tmp ) return -1;
    txt->text = tmp;
  }
  for( i = txt->last ; i >= txt->sel ; i-- ) {
    txt->text[i + l] = txt->text[i];
  }
  memcpy(&txt->text[txt->sel],s,l);
  txt->sel += l;
  txt->last += l;
  cct_textbox_movefirst(txt);
  return 0;
}

void cct_textbox_draw(cct_textbox *txt) {
  int n;
  char c;
  short attr;
  cct_hline(txt->t,txt->x,txt->y,' ',txt->w);
  cct_nputs(txt->t,txt->x,txt->y,&txt->text[txt->first],txt->w);
  attr = txt->t->attr;
  txt->t->attr = attr ^ CCT_REVERSE;
  /* 0123456789
   * abcdefghik
   *   |....|
   *     ^
   *     |
   *    sel
   */
  n = txt->sel - txt->first;
  c = txt->text[txt->sel];
  if ( c == 0 ) c = ' ';
  cct_putc(txt->t,txt->x + n,txt->y,c);
  txt->t->attr = attr;
}

int cct_textbox_keypress(cct_textbox *txt,int c) {
  if ( c == 0 ) return 0;
  switch(c) {
    case 0x02: /* CTRL+B */
    case CCT_KEY_LEFT:
      cct_textbox_prev(txt);
      return 0;
    case 0x06: /* CTRL+F */
    case CCT_KEY_RIGHT:
      cct_textbox_next(txt);
      return 0;
    case 0x01: /* CTRL+A */
      cct_textbox_home(txt);
      return 0;
    case 0x05: /* CTRL+E */
      cct_textbox_end(txt);
      return 0;
    case 0x08: /* BS */
    case 0x7F: /* DEBIAN BS */
      cct_textbox_bs(txt);
      return 0;
    case 0x04: /* CTRL+D */
      cct_textbox_del(txt);
      return 0;
    case 0x15: /* CTRL+U */
      cct_textbox_clear(txt);
      return 0;
    default:
      if ( c < 0 || c >= 0x20 ) {
        char s[2];
        s[0] = (char)c;
        s[1] = 0;
        return cct_textbox_insert(txt,s);
      }
  }
  return 0;
}

