/*
 * Copyright (C) 2010-2014  Guillaume Quintin.
 * This file is part of cocoju.
 *
 * cocoju is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void end(const char *msg,...) {
  if ( G.ccted ) cct_end_term(&G.term);
  if ( msg ) {
    va_list ap;
    va_start(ap,msg);
    vfprintf(stderr,msg,ap);
    fputc('\n',stderr);
    va_end(ap);
    exit(EXIT_FAILURE);
  }
  exit(EXIT_SUCCESS);
}

void *malloc2(uma n) {
  void *ret;
  ret = malloc(n);
  if ( !ret ) end("cannot malloc %d bytes\n",n);
  return ret;
}

void *realloc2(void *p,uma n) {
  void *ret;
  if ( !p ) return malloc2(n);
  ret = realloc(p,n);
  if ( !ret ) end("cannot realloc %d bytes at %X\n",n,p);
  return ret;
}

char *strdup2(const char *s) {
  char *t = malloc2(strlen(s) + 1);
  strcpy(t,s);
  if ( !t ) end("cannot strdup string at %X\n",s);
  return t;
}

char *strcat2(const char *s,const char *t) {
  uma ns,nt;
  char *u;
  ns = strlen(s);
  nt = strlen(t);
  u = malloc2(ns + nt + 1);
  strcpy(u,s);
  strcpy(&u[ns],t);
  return u;
}

void set_error(const char *s) {
  G.error = strcat2("ERROR: ",s);
}

char *path_resolution(const char *path) {
  char *pwd_path = getenv("PWD");
  char *absolute_path,*i,*j;

  if ( path[0] == '/' ) {
    absolute_path = strdup2(path);
  }
  else {
    absolute_path = malloc2(strlen(pwd_path) + strlen(path) + 2);
    sprintf(absolute_path,"%s/%s",pwd_path,path);
  }

  for( i = absolute_path,j = absolute_path ; j[0] ; ) {
    if ( j[0] == '/' && j[1] == 0 && i > absolute_path ) break;
    if ( j[0] == '/' && j[1] == '/' ) {
      j++;
      continue;
    }
    if ( j[0] == '/' && j[1] == '.' && (j[2] == '/' || j[2] == 0) ) {
      j += 2;
      continue;
    }
    if ( j[0] == '/' && j[1] == '.' && j[2] == '.' && 
         (j[3] == '/' || j[3] == 0) ) {
      if ( i > absolute_path ) {
        i--;
        for( ; i >= absolute_path && i[0] != '/' ; i-- );
      }
      j += 3;
      continue;
    }
    /* here, I am sure I am not in the case of a
       . or .. directory so I can copy until the next '/' */
    i[0] = j[0];
    for( i++,j++ ; j[0] && j[0] != '/' ; j++,i++ ) i[0] = j[0];
  }

  if ( i == absolute_path ) {
    absolute_path[0] = '/';
    absolute_path[1] = 0;
  }
  else {
    i[0] = 0;
  }
  return absolute_path;
}

char *parent_path(char *s) {
  uma i;
  if ( s[0] == 0 ) return 0;
  if ( s[0] == '/' && s[1] == 0 ) return s;
  i = strlen(s);
  i--;
  for( ; i >= 0 && s[i] != '/' ; i-- );
  if ( i < 0 ) return 0;
  s[i] = 0;
  return s;
}

char tolower2(char c) {
  if ( c < 'A' || c > 'Z' ) return c;
  return c + 'a' - 'A';
}

int hiddenfile(const char *file) {
  if ( file[0] != '.' ) return 0;
  if ( file[0] == '.' && file[1] == 0 ) return 1;
  if ( file[0] == '.' && file[1] == '.' && file[2] == 0 ) return 1;
  /* here: file[0] == '.' */
  if ( G.show_hidden ) return 0;
  return 1;
}

int alphasort2(const void *a,const void *b) {
  char *s = *(char**)a,*t = *(char**)b;
  for( ; s[0] && t[0] && tolower2(s[0]) == tolower2(t[0]) ; s++,t++ );
  return tolower2(s[0]) - tolower2(t[0]);
}

/* search for s within t */
char *strstr2(const char *s,const char *t)  {
  char c;
  const char *u,*v;
  for( ; t[0] ; ) {
    for( c = tolower2(s[0]) ; t[0] && c != tolower2(t[0]) ; t++ );
    if ( t[0] == 0 ) return 0;
    for( u = s,v = t ;
         u[0] && v[0] && tolower2(u[0]) == tolower2(v[0]) ;
         u++,v++ );
    if ( u[0] == 0 ) return (char*)t;
    t++;
  }
  return 0;
}

void killdir(void *entries,uma n) {
  void ***p;
  uma i;
  for( i = 0,p = (void***)entries ; i < n ; i++ ) free(p[i] - 1);
  free(entries);
}

uma max(uma a,uma b) {
  if ( a > b ) return a;
  return b;
}

void fifo_push(void **start,uma *n,uma sz,void *item) {
  void *ret;
  uma nn;

  nn = n[0] + 1;
  ret = realloc2(start[0],nn * sz);
  memcpy((void*)(((char*)ret) + n[0] * sz),item,sz);
  start[0] = ret;
  n[0] = nn;
}

void fifo_pop(void **start,uma *n,uma sz) {
  void *ret;
  uma nn;

  if ( n[0] <= 0 ) return;
  nn = n[0] - 1;
  ret = realloc2(start[0],nn * sz);
  start[0] = ret;
  n[0] = nn;
}
